@@ -4,7 +4,7 @@
 # 
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
-# Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
+# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
 # 
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,15 +19,22 @@
 # You should have received a copy of the GNU General Public License
 # along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
 #
-
+    
 import os
 import gettext as _gettext
 from locale import getdefaultlocale
 
-version = "0.13.1"
+# Hack to make RabbitVCS win in the battle against TortoiseHg
+try:
+    import mercurial.demandimport
+    mercurial.demandimport.enable = lambda: None
+except Exception, e:
+    pass
+
+version = "0.15.2"
 APP_NAME = "RabbitVCS"
 TEMP_DIR_PREFIX = "rabbitvcs-"
-LOCALE_DIR = "%s/locale" % os.path.dirname(os.path.abspath(__file__))
+LOCALE_DIR = "%s/locale" % os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 if not os.path.exists(LOCALE_DIR):
     LOCALE_DIR = "/usr/share/locale"
 
@@ -45,7 +52,6 @@ _gettext.textdomain(APP_NAME)
 
 gettext = _gettext.translation(APP_NAME, LOCALE_DIR, languages=langs, fallback=True)
 
-
 def package_name():
     """
     Report the application name in a form appropriate for building
@@ -72,3 +78,27 @@ def package_identifier():
 
     """
     return "%s-%s" % (package_name(), package_version())
+    
+def package_prefix():
+    """
+    Return the prefix of the local RabbitVCS installation
+    
+    """
+
+    try:
+        from rabbitvcs.buildinfo import rabbitvcs_prefix
+        return rabbitvcs_prefix
+    except ImportError, e:
+        return ""
+
+def get_icon_path():
+    """
+    Return the path to the icon folder
+    
+    """
+    
+    try:
+        from rabbitvcs.buildinfo import icon_path
+        return icon_path
+    except ImportError, e:
+        return "%s/data/icons/hicolor" % os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
