@@ -4,7 +4,7 @@
 # 
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
-# Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
+# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
 # 
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,16 +29,16 @@ import gtk
 from datetime import datetime
 
 from rabbitvcs.ui import InterfaceView
-from rabbitvcs.lib.contextmenu import GtkContextMenu, GtkContextMenuCaller, \
+from rabbitvcs.util.contextmenu import GtkContextMenu, GtkContextMenuCaller, \
     GtkFilesContextMenuConditions
-from rabbitvcs.lib.contextmenuitems import *
+from rabbitvcs.util.contextmenuitems import *
 import rabbitvcs.ui.widget
 import rabbitvcs.ui.dialog
 import rabbitvcs.ui.action
-import rabbitvcs.lib.helper
-import rabbitvcs.lib.vcs
-from rabbitvcs.lib.log import Log
-from rabbitvcs.lib.decorators import gtk_unsafe
+import rabbitvcs.util.helper
+import rabbitvcs.vcs
+from rabbitvcs.util.log import Log
+from rabbitvcs.util.decorators import gtk_unsafe
 
 log = Log("rabbitvcs.ui.browser")
 
@@ -47,19 +47,26 @@ _ = gettext.gettext
 
 gtk.gdk.threads_init()
 
-class Browser(InterfaceView, GtkContextMenuCaller):
+class SVNBrowser(InterfaceView, GtkContextMenuCaller):
     def __init__(self, url):
         InterfaceView.__init__(self, "browser", "Browser")
 
-        self.vcs = rabbitvcs.lib.vcs.create_vcs_instance()
-        self.url = self.vcs.get_repo_url(url)
+        self.vcs = rabbitvcs.vcs.VCS()
+        self.svn = self.vcs.svn()
+
+        self.url = ""
+        if self.svn.is_in_a_or_a_working_copy(url):
+            action = rabbitvcs.ui.action.SVNAction(self.svn, notification=False, run_in_thread=False)
+            self.url = action.run_single(self.svn.get_repo_url, url)
+        elif self.svn.is_path_repository_url(url):
+            self.url = url
 
         self.urls = rabbitvcs.ui.widget.ComboBox(
             self.get_widget("urls"), 
-            rabbitvcs.lib.helper.get_repository_paths()
+            rabbitvcs.util.helper.get_repository_paths()
         )
         if self.url:
-            self.urls.set_child_text(self.url)
+            self.urls.set_child_text(rabbitvcs.util.helper.unquote_url(self.url))
 
         # We must set a signal handler for the gtk.Entry inside the combobox
         # Because glade will not retain that information
@@ -70,7 +77,7 @@ class Browser(InterfaceView, GtkContextMenuCaller):
 
         self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
             self.get_widget("revision_container"),
-            self.vcs,
+            self.svn,
             url_combobox=self.urls
         )
 
@@ -107,6 +114,9 @@ class Browser(InterfaceView, GtkContextMenuCaller):
                 "file-column-callback": self.file_column_callback,
                 "row-activated": self.on_row_activated,
                 "mouse-event":   self.on_list_table_mouse_event
+            },
+            flags={
+                "sortable": True
             }
         )
         
@@ -114,18 +124,21 @@ class Browser(InterfaceView, GtkContextMenuCaller):
         self.url_clipboard = gtk.Clipboard()
         self.repo_root_url = None
 
-        if url:
-            rabbitvcs.lib.helper.save_repository_path(url)
+        if self.url:
+            rabbitvcs.util.helper.save_repository_path(url)
             self.load()
 
     def load(self):
-        self.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs,
+        self.url = self.urls.get_active_text()
+        self.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
         revision = self.revision_selector.get_revision_object()
-        self.action.append(self.vcs.list, self.urls.get_active_text(), 
-            revision=revision, recurse=False)
+        self.action.append(
+                        self.svn.list,
+                        rabbitvcs.util.helper.quote_url(self.url), 
+                        revision=revision, recurse=False)
         self.action.append(self.init_repo_root_url)
         self.action.append(self.populate_table, 0)
         self.action.start()
@@ -147,8 +160,9 @@ class Browser(InterfaceView, GtkContextMenuCaller):
             ])
     
     def init_repo_root_url(self):
-        if self.repo_root_url is None:
-            self.repo_root_url = self.vcs.get_repo_root_url(self.url)
+        if self.repo_root_url is None and self.svn.is_in_a_or_a_working_copy(self.url):
+            action = rabbitvcs.ui.action.SVNAction(self.svn, notification=False, run_in_thread=False)
+            self.repo_root_url = action.run_single(self.svn.get_repo_root_url, self.url)
     
     def on_destroy(self, widget):
         self.destroy()
@@ -157,6 +171,7 @@ class Browser(InterfaceView, GtkContextMenuCaller):
         self.close()
 
     def on_refresh_clicked(self, widget):
+        rabbitvcs.util.helper.save_repository_path(self.urls.get_active_text())
         self.load()
 
     def on_row_activated(self, treeview, data, col):
@@ -167,11 +182,15 @@ class Browser(InterfaceView, GtkContextMenuCaller):
         else:
             self.url = path
 
-        self.urls.set_child_text(self.url)
-        self.load()
+        if self.file_column_callback(self.url) == "dir" or self.url != path:
+            self.urls.set_child_text(rabbitvcs.util.helper.unquote_url(self.url))
+            self.load()
+        else:
+            self._open([self.url])
 
     def on_urls_key_released(self, widget, data, userdata):
         if gtk.gdk.keyval_name(data.keyval) == "Return":
+            rabbitvcs.util.helper.save_repository_path(self.urls.get_active_text())
             self.load()
 
     def file_column_callback(self, filename):
@@ -184,7 +203,7 @@ class Browser(InterfaceView, GtkContextMenuCaller):
 
         for item,locked in self.items:
             if item.path == filename:
-                return self.vcs.NODE_KINDS_REVERSE[item.kind]
+                return self.svn.NODE_KINDS_REVERSE[item.kind]
         return None
 
     def sort_files(self, x, y):
@@ -193,8 +212,8 @@ class Browser(InterfaceView, GtkContextMenuCaller):
         alphabetically.
 
         """
-        xkind = self.vcs.NODE_KINDS_REVERSE[x[0].kind]
-        ykind = self.vcs.NODE_KINDS_REVERSE[y[0].kind]
+        xkind = self.svn.NODE_KINDS_REVERSE[x[0].kind]
+        ykind = self.svn.NODE_KINDS_REVERSE[y[0].kind]
         if xkind == "dir" and ykind == "dir":
             return cmp(x[0].repos_path, y[0].repos_path)
         elif xkind == "dir" and ykind == "file":
@@ -218,7 +237,7 @@ class Browser(InterfaceView, GtkContextMenuCaller):
         """
 
         if self.file_column_callback(row[0]) == "file":
-            return rabbitvcs.lib.helper.pretty_filesize(row[column])
+            return rabbitvcs.util.helper.pretty_filesize(int(row[column]))
 
         return ""
 
@@ -241,8 +260,8 @@ class Browser(InterfaceView, GtkContextMenuCaller):
             return ""
         
         if row[column]:
-            change_time = datetime.fromtimestamp(row[column])
-            return change_time.strftime("%Y-%m-%d %H:%M:%S")
+            change_time = datetime.fromtimestamp(float(row[column]))
+            return rabbitvcs.util.helper.format_datetime(change_time)
         
         return str(row[column])
 
@@ -281,15 +300,37 @@ class Browser(InterfaceView, GtkContextMenuCaller):
     def get_url(self):
         return self.urls.get_active_text()
 
-class BrowserDialog(Browser):
+    def _open(self, paths):
+        self.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
+            notification=False
+        )
+        
+        exported_paths = []
+        for path in paths:
+            export_path = rabbitvcs.util.helper.get_tmp_path(os.path.basename(paths[0]))
+            exported_paths.append(export_path)
+            self.action.append(self.svn.export, paths[0], 
+                export_path, revision=self.revision_selector.get_revision_object())
+
+        for path in exported_paths:
+            self.action.append(rabbitvcs.util.helper.open_item, path)
+
+        self.action.start()
+
+class SVNBrowserDialog(SVNBrowser):
     def __init__(self, path, callback=None):
         """
         Override the normal Browser class so that we can hide the window as we need.
         Also, provide a callback for when the close button is clicked so that we
         can get some desired data.
         """
-        Browser.__init__(self, path)
+        
         self.callback = callback
+
+        gtk.stock_add([(gtk.STOCK_CLOSE, _("Select"), 0, 0, "")])
+        
+        SVNBrowser.__init__(self, path)
         
     def on_destroy(self, widget):
         pass
@@ -300,7 +341,7 @@ class BrowserDialog(Browser):
             path = self.urls.get_active_text()
             selected = self.list_table.get_selected_row_items(0)
             if len(selected) > 0:
-                path = selected
+                path = selected[0]
             self.callback(path)
 
 
@@ -328,8 +369,8 @@ class MenuBrowserMoveTo(MenuItem):
 
 
 class BrowserContextMenuConditions(GtkFilesContextMenuConditions):
-    def __init__(self, vcs_client, paths, caller):
-        GtkFilesContextMenuConditions.__init__(self, vcs_client, paths)
+    def __init__(self, vcs, paths, caller):
+        GtkFilesContextMenuConditions.__init__(self, vcs, paths)
         self.caller = caller
 
     def _open(self, data1=None, data2=None):
@@ -375,11 +416,13 @@ class BrowserContextMenuConditions(GtkFilesContextMenuConditions):
         return (revision.kind == "head")
 
 class BrowserContextMenuCallbacks:
-    def __init__(self, caller, base_dir, vcs_client, paths=[]):
+    def __init__(self, caller, base_dir, vcs, paths=[]):
         self.caller = caller
         self.base_dir = base_dir
-        self.vcs_client = vcs_client
+        self.vcs = vcs
+        self.svn = self.vcs.svn()
         self.paths = paths
+        self.guess = rabbitvcs.vcs.VCS_SVN
 
     def __update_browser_url(self, url):
         # Make sure the Browser variables are updated with the new path
@@ -400,46 +443,31 @@ class BrowserContextMenuCallbacks:
         return sources
 
     def _open(self, data=None, user_data=None):
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
-            notification=False
-        )
-        
-        exported_paths = []
-        for path in self.paths:
-            export_path = "/tmp/" + os.path.basename(self.paths[0])
-            exported_paths.append(export_path)
-            self.caller.action.append(self.vcs_client.export, self.paths[0], 
-                export_path, revision=self.__get_browser_revision())
-
-        for path in exported_paths:
-            self.caller.action.append(rabbitvcs.lib.helper.open_item, path)
-
-        self.caller.action.start()
+        self.caller._open(self.paths)
     
     def show_log(self, data=None, user_data=None):
-        rabbitvcs.lib.helper.launch_ui_window("log", [self.paths[0]])
+        rabbitvcs.util.helper.launch_ui_window("log", ["--vcs=%s" % self.guess, self.paths[0]])
     
     def annotate(self, data=None, user_data=None):
         urlrev = self.paths[0]
         revision = self.__get_browser_revision()
         if revision.kind == "number":
             urlrev += "@" + revision.value
-        rabbitvcs.lib.helper.launch_ui_window("annotate", [urlrev])
+        rabbitvcs.util.helper.launch_ui_window("annotate", [urlrev])
     
     def checkout(self, data=None, user_data=None):
         args = [self.paths[0]]
         revision = self.__get_browser_revision()
         if revision.kind == "number":
             args = ["-r", revision.value] + args
-        rabbitvcs.lib.helper.launch_ui_window("checkout", args)
+        rabbitvcs.util.helper.launch_ui_window("checkout", args)
     
     def export(self, data=None, user_data=None):
         args = [self.paths[0]]
         revision = self.__get_browser_revision()
         if revision.kind == "number":
             args = ["-r", revision.value] + args
-        rabbitvcs.lib.helper.launch_ui_window("export", args)
+        rabbitvcs.util.helper.launch_ui_window("export", args)
         
     def rename(self, data=None, user_data=None):
         (base, filename) = os.path.split(self.paths[0])
@@ -457,12 +485,12 @@ class BrowserContextMenuCallbacks:
             path_to_refresh = new_url
             self.__update_browser_url(path_to_refresh)
 
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
+        self.caller.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
-        self.caller.action.append(self.vcs_client.move, self.paths[0], new_url)
-        self.caller.action.append(self.vcs_client.list, path_to_refresh, recurse=False)
+        self.caller.action.append(self.svn.move, self.paths[0], new_url)
+        self.caller.action.append(self.svn.list, path_to_refresh, recurse=False)
         self.caller.action.append(self.caller.populate_table, 1)
         self.caller.action.start()
     
@@ -475,12 +503,12 @@ class BrowserContextMenuCallbacks:
         
             self.__update_browser_url(path_to_refresh)
         
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
+        self.caller.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
-        self.caller.action.append(self.vcs_client.remove, self.paths)
-        self.caller.action.append(self.vcs_client.list, path_to_refresh, recurse=False)
+        self.caller.action.append(self.svn.remove, self.paths)
+        self.caller.action.append(self.svn.list, path_to_refresh, recurse=False)
         self.caller.action.append(self.caller.populate_table, 1)
         self.caller.action.start()  
 
@@ -494,12 +522,12 @@ class BrowserContextMenuCallbacks:
         (folder_name, log_message) = result
         new_url = self.paths[0].rstrip("/") + "/" + folder_name
 
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
+        self.caller.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
-        self.caller.action.append(self.vcs_client.mkdir, new_url, log_message)
-        self.caller.action.append(self.vcs_client.list, self.paths[0], recurse=False)
+        self.caller.action.append(self.svn.mkdir, new_url, log_message)
+        self.caller.action.append(self.svn.list, self.paths[0], recurse=False)
         self.caller.action.append(self.caller.populate_table, 1)
         self.caller.action.start()        
 
@@ -520,12 +548,12 @@ class BrowserContextMenuCallbacks:
 
         sources = self.__generate_sources_list()
 
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
+        self.caller.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
-        self.caller.action.append(self.vcs_client.copy_all, sources, new_url, copy_as_child=True)
-        self.caller.action.append(self.vcs_client.list, self.caller.get_url(), recurse=False)
+        self.caller.action.append(self.svn.copy_all, sources, new_url, copy_as_child=True)
+        self.caller.action.append(self.svn.list, self.caller.get_url(), recurse=False)
         self.caller.action.append(self.caller.populate_table, 1)
         self.caller.action.start()
 
@@ -547,33 +575,34 @@ class BrowserContextMenuCallbacks:
         if response == gtk.RESPONSE_CANCEL:
             return
 
-        self.caller.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs_client,
+        self.caller.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
-        self.caller.action.append(self.vcs_client.move_all, self.paths, new_url, move_as_child=True)
-        self.caller.action.append(self.vcs_client.list, self.caller.get_url(), recurse=False)
+        self.caller.action.append(self.svn.move_all, self.paths, new_url, move_as_child=True)
+        self.caller.action.append(self.svn.list, self.caller.get_url(), recurse=False)
         self.caller.action.append(self.caller.populate_table, 1)
         self.caller.action.start()
 
 class BrowserContextMenu:
-    def __init__(self, caller, event, base_dir, vcs_client, paths=[]):
+    def __init__(self, caller, event, base_dir, vcs, paths=[]):
         
         self.caller = caller
         self.event = event
         self.paths = paths
         self.base_dir = base_dir
-        self.vcs_client = vcs_client
+        self.vcs = vcs
+        self.svn = self.vcs.svn()
         
         self.conditions = BrowserContextMenuConditions(
-            self.vcs_client, 
+            self.vcs, 
             paths,
             self.caller
         )
         self.callbacks = BrowserContextMenuCallbacks(
             self.caller, 
             self.base_dir,
-            self.vcs_client, 
+            self.vcs, 
             paths
         )
 
@@ -601,12 +630,21 @@ class BrowserContextMenu:
         context_menu = GtkContextMenu(self.structure, self.conditions, self.callbacks)
         context_menu.show(self.event)
 
+classes_map = {
+    rabbitvcs.vcs.VCS_SVN: SVNBrowser,
+    rabbitvcs.vcs.VCS_DUMMY: SVNBrowser
+}
+
+def browser_factory(path):
+    guess = rabbitvcs.vcs.guess(path)
+    return classes_map[guess["vcs"]](path)
+
 if __name__ == "__main__":
     from rabbitvcs.ui import main
     (options, url) = main(
         usage="Usage: rabbitvcs browser [url]"
     )
 
-    window = Browser(url[0])
+    window = browser_factory(url[0])
     window.register_gtk_quit()
     gtk.main()
