@@ -4,7 +4,7 @@
 # 
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
-# Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
+# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
 # 
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,11 +27,10 @@ import gtk
 
 from rabbitvcs.ui import InterfaceView
 import rabbitvcs.ui.widget
-import rabbitvcs.lib.helper
-from rabbitvcs.lib.contextmenu import GtkContextMenu
-from rabbitvcs.lib.contextmenuitems import *
-from rabbitvcs.ui.log import LogDialog
-from rabbitvcs.ui.action import VCSAction
+import rabbitvcs.util.helper
+from rabbitvcs.util.contextmenu import GtkContextMenu
+from rabbitvcs.util.contextmenuitems import *
+import rabbitvcs.ui.action
 from rabbitvcs.ui.dialog import MessageBox
 from rabbitvcs import gettext
 _ = gettext.gettext
@@ -54,7 +53,7 @@ class Changes(InterfaceView):
     def __init__(self, path1=None, revision1=None, path2=None, revision2=None):
         InterfaceView.__init__(self, "changes", "Changes")
         
-        self.vcs = rabbitvcs.lib.vcs.create_vcs_instance()
+        self.vcs = rabbitvcs.vcs.VCS()
 
         self.MORE_ACTIONS_CALLBACKS = [
             None,
@@ -67,7 +66,7 @@ class Changes(InterfaceView):
         )
         self.more_actions.set_active(0)
 
-        repo_paths = rabbitvcs.lib.helper.get_repository_paths()
+        repo_paths = rabbitvcs.util.helper.get_repository_paths()
         self.first_urls = rabbitvcs.ui.widget.ComboBox(
             self.get_widget("first_urls"), 
             repo_paths
@@ -79,41 +78,7 @@ class Changes(InterfaceView):
             repo_paths
         )
         self.second_urls_browse = self.get_widget("second_urls_browse")
-
-        self.first_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
-            self.get_widget("first_revision_container"),
-            self.vcs,
-            revision=revision1,
-            url_combobox=self.first_urls
-        )
-
-        self.second_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
-            self.get_widget("second_revision_container"),
-            self.vcs,
-            revision=revision2,
-            url_combobox=self.second_urls
-        )
-
-        if path1 is not None:
-            self.first_urls.set_child_text(self.vcs.get_repo_url(path1))
-            
-        if path2 is not None:
-            self.second_urls.set_child_text(self.vcs.get_repo_url(path2))
-        elif path1 is not None:
-            self.second_urls.set_child_text(self.vcs.get_repo_url(path1))
-
-        self.changes_table = rabbitvcs.ui.widget.Table(
-            self.get_widget("changes_table"),
-            [rabbitvcs.ui.widget.TYPE_PATH, gobject.TYPE_STRING, 
-                gobject.TYPE_STRING], 
-            [_("Path"), _("Change"), _("Property Change")]
-        )
         
-        self.check_ui()
-        
-        if path1 and revision1 and path2 and revision2:
-            self.load()
-
     #
     # UI Signal Callback Methods
     #
@@ -129,34 +94,26 @@ class Changes(InterfaceView):
     
     def on_first_urls_changed(self, widget, data=None):
         self.check_first_urls()
-        self.first_revision_selector.determine_widget_sensitivity()
         self.check_refresh_button()
 
     def on_second_urls_changed(self, widget, data=None):
         self.check_second_urls()
-        self.second_revision_selector.determine_widget_sensitivity()
         self.check_refresh_button()
 
     def on_first_urls_browse_clicked(self, widget, data=None):
-        from rabbitvcs.ui.browser import BrowserDialog
-        BrowserDialog(self.first_urls.get_active_text(), 
-            callback=self.on_first_repo_chooser_closed)
+        pass
 
     def on_first_repo_chooser_closed(self, new_url):
         self.first_urls.set_child_text(new_url)
         self.check_first_urls()
-        self.first_revision_selector.determine_widget_sensitivity()
         self.check_refresh_button()
 
     def on_second_urls_browse_clicked(self, widget, data=None):
-        from rabbitvcs.ui.browser import BrowserDialog
-        BrowserDialog(self.second_urls.get_active_text(), 
-            callback=self.on_second_repo_chooser_closed)
+        pass
 
     def on_second_repo_chooser_closed(self, new_url):
         self.second_urls.set_child_text(new_url)
         self.check_second_urls()
-        self.second_revision_selector.determine_widget_sensitivity()
         self.check_refresh_button()
 
     def on_changes_table_cursor_changed(self, treeview, data=None):
@@ -194,7 +151,7 @@ class Changes(InterfaceView):
         for tup in indexes:
             self.selected_rows.append(tup[0])
 
-        self.view_selected_diff()
+        self.view_selected_diff(sidebyside=True)
 
     #
     # Helper methods
@@ -212,8 +169,6 @@ class Changes(InterfaceView):
     def check_ui(self):
         self.check_first_urls()
         self.check_second_urls()
-        self.first_revision_selector.determine_widget_sensitivity()
-        self.second_revision_selector.determine_widget_sensitivity()
         self.check_refresh_button()
     
     def can_first_browse_urls(self):
@@ -238,26 +193,122 @@ class Changes(InterfaceView):
         can_browse_urls = self.can_second_browse_urls()
         self.second_urls_browse.set_sensitive(can_browse_urls)
 
+
+    def enable_more_actions(self):
+        self.more_actions.set_sensitive(True)
+
+    def disable_more_actions(self):
+        self.more_actions.set_sensitive(False)
+    
+    def view_selected_diff(self, sidebyside=False):
+        url1 = self.changes_table.get_row(self.selected_rows[0])[0]
+        url2 = url1
+        if url1 == ".":
+            url1 = ""
+            url2 = ""
+
+        url1 = rabbitvcs.util.helper.url_join(self.first_urls.get_active_text(), url1)
+        url2 = rabbitvcs.util.helper.url_join(self.second_urls.get_active_text(), url2)
+        rev1 = self.get_first_revision()
+        rev2 = self.get_second_revision()
+        
+        rabbitvcs.util.helper.launch_ui_window("diff", [
+            "%s@%s" % (url1, unicode(rev1)),
+            "%s@%s" % (url2, unicode(rev2)),
+            "%s" % (sidebyside and "-s" or ""),
+            "--vcs=%s" % self.get_vcs_name()
+        ])
+        
+        
+    #
+    # More Actions callbacks
+    #
+
+    def on_more_actions_view_unified_diff(self):
+        url1 = self.first_urls.get_active_text()
+        rev1 = self.get_first_revision()
+        rev2 = self.get_second_revision()        
+        url2 = self.second_urls.get_active_text()
+
+        rabbitvcs.util.helper.launch_ui_window("diff", [
+            "%s@%s" % (url1, unicode(rev1)),
+            "%s@%s" % (url2, unicode(rev2)),
+            "--vcs=%s" % self.get_vcs_name()
+        ])
+
+    def get_vcs_name(self):
+        vcs = rabbitvcs.vcs.VCS_DUMMY
+        if hasattr(self, "svn"):
+            vcs = rabbitvcs.vcs.VCS_SVN
+        elif hasattr(self, "git"):
+            vcs = rabbitvcs.vcs.VCS_GIT
+
+        return vcs
+
+class SVNChanges(Changes):
+    def __init__(self, path1=None, revision1=None, path2=None, revision2=None):
+        Changes.__init__(self, path1, revision1, path2, revision2)
+
+        self.svn = self.vcs.svn()
+
+        if path1 is not None:
+            self.first_urls.set_child_text(self.svn.get_repo_url(path1))
+            
+        if path2 is not None:
+            self.second_urls.set_child_text(self.svn.get_repo_url(path2))
+        elif path1 is not None:
+            self.second_urls.set_child_text(self.svn.get_repo_url(path1))
+
+        self.first_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
+            self.get_widget("first_revision_container"),
+            self.svn,
+            revision=revision1,
+            url_combobox=self.first_urls
+        )
+
+        self.second_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
+            self.get_widget("second_revision_container"),
+            self.svn,
+            revision=revision2,
+            url_combobox=self.second_urls
+        )
+
+        self.changes_table = rabbitvcs.ui.widget.Table(
+            self.get_widget("changes_table"),
+            [gobject.TYPE_STRING, gobject.TYPE_STRING, 
+                gobject.TYPE_STRING], 
+            [_("Path"), _("Change"), _("Property Change")],
+            flags={
+                "sortable": True, 
+                "sort_on": 1
+            }
+        )
+
+        self.check_ui()
+        
+        if path1 and revision1 and path2 and revision2:
+            self.load()
+
     def load(self):
         first_url = self.first_urls.get_active_text()
         first_rev = self.get_first_revision()
         second_rev = self.get_second_revision()        
         second_url = self.second_urls.get_active_text()
 
-        self.action = VCSAction(
-            self.vcs,
+        self.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             notification=False
         )
         self.action.append(self.disable_more_actions)
         self.action.append(
-            self.vcs.diff_summarize,
+            self.svn.diff_summarize,
             first_url,
             first_rev,
             second_url,
             second_rev
         )
-        self.action.append(rabbitvcs.lib.helper.save_repository_path, first_url)
-        self.action.append(rabbitvcs.lib.helper.save_repository_path, second_url)
+        self.action.append(rabbitvcs.util.helper.save_repository_path, first_url)
+        self.action.append(rabbitvcs.util.helper.save_repository_path, second_url)
         self.action.append(self.populate_table)
         self.action.append(self.enable_more_actions)
         self.action.start()
@@ -280,77 +331,92 @@ class Changes(InterfaceView):
                 prop_changed
             ])
 
-    def enable_more_actions(self):
-        self.more_actions.set_sensitive(True)
+    def on_first_urls_browse_clicked(self, widget, data=None):
+        from rabbitvcs.ui.browser import SVNBrowserDialog
+        SVNBrowserDialog(self.first_urls.get_active_text(), 
+            callback=self.on_first_repo_chooser_closed)
 
-    def disable_more_actions(self):
-        self.more_actions.set_sensitive(False)
+    def on_second_urls_browse_clicked(self, widget, data=None):
+        from rabbitvcs.ui.browser import SVNBrowserDialog
+        SVNBrowserDialog(self.second_urls.get_active_text(), 
+            callback=self.on_second_repo_chooser_closed)
 
-    def open_item_from_revision(self, url, revision, dest):
-        self.action = VCSAction(
-            self.vcs,
-            notification=False
-        )
-        self.action.append(
-            self.vcs.export,
-            url,
-            dest,
-            revision=revision
-        )
-        self.action.append(rabbitvcs.lib.helper.open_item, dest)
-        self.action.start()
-    
-    def view_selected_diff(self):
-        from rabbitvcs.ui.diff import SVNDiff
-        url1 = self.changes_table.get_row(self.selected_rows[0])[0]
-        url2 = url1
-        if url1 == ".":
-            url1 = ""
-            url2 = ""
+class GitChanges(Changes):
+    def __init__(self, path1=None, revision1=None, path2=None, revision2=None):
+        Changes.__init__(self, path1, revision1, path2, revision2)
 
-        url1 = rabbitvcs.lib.helper.url_join(self.first_urls.get_active_text(), url1)
-        url2 = rabbitvcs.lib.helper.url_join(self.second_urls.get_active_text(), url2)
-        rev1 = self.get_first_revision()
-        rev2 = self.get_second_revision()
-
-        self.action = VCSAction(
-            self.vcs,
-            notification=False
-        )
-        self.action.append(
-            SVNDiff,
-            url1, 
-            (rev1.value and rev1.value or "HEAD"), 
-            url2, 
-            (rev2.value and rev2.value or "HEAD")
-        )
-        self.action.start()
+        self.git = self.vcs.git(path1)
         
-    #
-    # More Actions callbacks
-    #
+        self.first_urls_browse.hide()
+        self.second_urls_browse.hide()
 
-    def on_more_actions_view_unified_diff(self):
-        from rabbitvcs.ui.diff import SVNDiff
+        if path1 is not None:
+            self.first_urls.set_child_text(path1)
+            
+        if path2 is not None:
+            self.second_urls.set_child_text(path2)
+        elif path1 is not None:
+            self.second_urls.set_child_text(path1)
+
+        self.first_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
+            self.get_widget("first_revision_container"),
+            self.git,
+            revision=revision1,
+            url_combobox=self.first_urls
+        )
+
+        self.second_revision_selector = rabbitvcs.ui.widget.RevisionSelector(
+            self.get_widget("second_revision_container"),
+            self.git,
+            revision=revision2,
+            url_combobox=self.second_urls
+        )
+
+        self.changes_table = rabbitvcs.ui.widget.Table(
+            self.get_widget("changes_table"),
+            [gobject.TYPE_STRING, gobject.TYPE_STRING], 
+            [_("Path"), _("Change")]
+        )
+
+        self.check_ui()
         
+        if path1 and revision1 and path2 and revision2:
+            self.load()
+
+    def load(self):
         first_url = self.first_urls.get_active_text()
         first_rev = self.get_first_revision()
         second_rev = self.get_second_revision()        
         second_url = self.second_urls.get_active_text()
 
-        self.action = VCSAction(
-            self.vcs,
+        self.action = rabbitvcs.ui.action.GitAction(
+            self.git,
             notification=False
         )
+        self.action.append(self.disable_more_actions)
         self.action.append(
-            SVNDiff,
-            first_url, 
-            (first_rev.value and first_rev.value or "HEAD"), 
-            second_url, 
-            (second_rev.value and second_rev.value or "HEAD")
+            self.git.diff_summarize,
+            first_url,
+            first_rev,
+            second_url,
+            second_rev
         )
+        self.action.append(rabbitvcs.util.helper.save_repository_path, first_url)
+        self.action.append(rabbitvcs.util.helper.save_repository_path, second_url)
+        self.action.append(self.populate_table)
+        self.action.append(self.enable_more_actions)
         self.action.start()
 
+    def populate_table(self):
+        # returns a list of dicts(path, summarize_kind, node_kind, prop_changed)
+        summary = self.action.get_result(1)
+
+        self.changes_table.clear()
+        for item in summary:
+            self.changes_table.append([
+                item.path,
+                item.action
+            ])
 
 
 class MenuOpenFirst(MenuItem):
@@ -374,9 +440,9 @@ class MenuCompare(MenuItem):
     icon = "rabbitvcs-compare"
 
 class ChangesContextMenuConditions:
-    def __init__(self, caller, vcs_client):
+    def __init__(self, caller, vcs):
         self.caller = caller
-        self.vcs_client = vcs_client
+        self.vcs = vcs
 
     def open_first(self):
         return (
@@ -403,29 +469,27 @@ class ChangesContextMenuConditions:
         )
 
 class ChangesContextMenuCallbacks:
-    def __init__(self, caller, vcs_client):
+    def __init__(self, caller, vcs):
         self.caller = caller
-        self.vcs_client = vcs_client
+        self.vcs = vcs
 
     def open_first(self, widget, data=None):
         path = self.caller.changes_table.get_row(self.caller.selected_rows[0])[0]
         if path == ".":
             path = ""
 
-        url = rabbitvcs.lib.helper.url_join(self.caller.first_urls.get_active_text(), path)
+        url = rabbitvcs.util.helper.url_join(self.caller.first_urls.get_active_text(), path)
         rev = self.caller.get_first_revision()
-        dest = "/tmp/rabbitvcs-" + str(rev) + "-" + os.path.basename(url)
-        self.caller.open_item_from_revision(url, rev, dest)
+        rabbitvcs.util.helper.launch_ui_window("open", [url, "-r", unicode(rev)])        
 
     def open_second(self, widget, data=None):
         path = self.caller.changes_table.get_row(self.caller.selected_rows[0])[0]
         if path == ".":
             path = ""
-        
-        url = rabbitvcs.lib.helper.url_join(self.caller.second_urls.get_active_text(), path)
+
+        url = rabbitvcs.util.helper.url_join(self.caller.second_urls.get_active_text(), path)
         rev = self.caller.get_second_revision()
-        dest = "/tmp/rabbitvcs-" + str(rev) + "-" + os.path.basename(url)
-        self.caller.open_item_from_revision(url, rev, dest)
+        rabbitvcs.util.helper.launch_ui_window("open", [url, "-r", unicode(rev)])
 
     def view_diff(self, widget, data=None):
         self.caller.view_selected_diff()
@@ -437,25 +501,17 @@ class ChangesContextMenuCallbacks:
             url1 = ""
             url2 = ""
 
-        url1 = rabbitvcs.lib.helper.url_join(self.caller.first_urls.get_active_text(), url1)
-        url2 = rabbitvcs.lib.helper.url_join(self.caller.second_urls.get_active_text(), url2)
+        url1 = rabbitvcs.util.helper.url_join(self.caller.first_urls.get_active_text(), url1)
+        url2 = rabbitvcs.util.helper.url_join(self.caller.second_urls.get_active_text(), url2)
         rev1 = self.caller.get_first_revision()
         rev2 = self.caller.get_second_revision()
         
-        from rabbitvcs.ui.diff import SVNDiff
-        self.action = VCSAction(
-            self.vcs_client,
-            notification=False
-        )
-        self.action.append(
-            SVNDiff,
-            url2,
-            rev2,
-            url1,
-            rev1,
-            sidebyside=True
-        )
-        self.action.start()
+        rabbitvcs.util.helper.launch_ui_window("diff", [
+            "%s@%s" % (url1, unicode(rev1)), 
+            "%s@%s" % (url2, unicode(rev2)), 
+            "-s",
+            "--vcs=%s" % self.caller.get_vcs_name()
+        ])
 
 class ChangesContextMenu:
     """
@@ -470,16 +526,17 @@ class ChangesContextMenu:
         """        
         self.caller = caller
         self.event = event
-        self.vcs_client = rabbitvcs.lib.vcs.create_vcs_instance()
+        self.vcs = rabbitvcs.vcs.VCS()
+        self.svn = self.vcs.svn()
 
         self.conditions = ChangesContextMenuConditions(
             self.caller,
-            self.vcs_client
+            self.vcs
         )
         
         self.callbacks = ChangesContextMenuCallbacks(
             self.caller,
-            self.vcs_client
+            self.vcs
         )
 
         # The first element of each tuple is a key that matches a
@@ -499,18 +556,30 @@ class ChangesContextMenu:
         context_menu = GtkContextMenu(self.structure, self.conditions, self.callbacks)
         context_menu.show(self.event)
 
+classes_map = {
+    rabbitvcs.vcs.VCS_SVN: SVNChanges,
+    rabbitvcs.vcs.VCS_GIT: GitChanges
+}
+
+def changes_factory(vcs, path1=None, revision1=None, path2=None, revision2=None):
+    if not vcs:
+        guess = rabbitvcs.vcs.guess(path1)
+        vcs = guess["vcs"]
+        
+    return classes_map[vcs](path1, revision1, path2, revision2)
 
 if __name__ == "__main__":
-    from rabbitvcs.ui import main
+    from rabbitvcs.ui import main, VCS_OPT
     (options, args) = main(
+        [VCS_OPT],
         usage="Usage: rabbitvcs changes [url1@rev1] [url2@rev2]"
     )
     
-    pathrev1 = rabbitvcs.lib.helper.parse_path_revision_string(args.pop(0))
+    pathrev1 = rabbitvcs.util.helper.parse_path_revision_string(args.pop(0))
     pathrev2 = (None, None)
     if len(args) > 0:
-        pathrev2 = rabbitvcs.lib.helper.parse_path_revision_string(args.pop(0))
+        pathrev2 = rabbitvcs.util.helper.parse_path_revision_string(args.pop(0))
 
-    window = Changes(pathrev1[0], pathrev1[1], pathrev2[0], pathrev2[1])
+    window = changes_factory(options.vcs, pathrev1[0], pathrev1[1], pathrev2[0], pathrev2[1])
     window.register_gtk_quit()
     gtk.main()
