@@ -4,7 +4,7 @@
 # 
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
-# Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
+# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
 # 
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,13 +28,12 @@ import gobject
 import gtk
 
 from rabbitvcs.ui import InterfaceView
-from rabbitvcs.ui.log import LogDialog
 import rabbitvcs.ui.widget
 import rabbitvcs.ui.dialog
 import rabbitvcs.ui.action
-import rabbitvcs.lib.helper
-import rabbitvcs.lib.vcs
-
+import rabbitvcs.util.helper
+import rabbitvcs.vcs
+from rabbitvcs.ui.updateto import GitUpdateToRevision
 from rabbitvcs import gettext
 _ = gettext.gettext
 
@@ -48,16 +47,13 @@ class Checkout(InterfaceView):
 
     def __init__(self, path=None, url=None, revision=None):
         InterfaceView.__init__(self, "checkout", "Checkout")
-
-
-        self.get_widget("Checkout").set_title(_("Checkout - %s") % path)
         
         self.path = path
-        self.vcs = rabbitvcs.lib.vcs.create_vcs_instance()
+        self.vcs = rabbitvcs.vcs.VCS()
 
         self.repositories = rabbitvcs.ui.widget.ComboBox(
             self.get_widget("repositories"), 
-            rabbitvcs.lib.helper.get_repository_paths()
+            rabbitvcs.util.helper.get_repository_paths()
         )
         
         # We must set a signal handler for the gtk.Entry inside the combobox
@@ -67,24 +63,15 @@ class Checkout(InterfaceView):
             self.on_repositories_key_released
         )
 
-        self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
-            self.get_widget("revision_container"),
-            self.vcs,
-            revision=revision,
-            url_combobox=self.repositories,
-            expand=True
-        )
-        
-        self.destination = rabbitvcs.lib.helper.get_user_path()
+        self.destination = rabbitvcs.util.helper.get_user_path()
         if path is not None:
             self.destination = path
             self.get_widget("destination").set_text(path)
-        
+
         if url is not None:
             self.repositories.set_child_text(url)
         
         self.complete = False
-        self.check_form()
         
     #
     # UI Signal Callback Methods
@@ -106,65 +93,12 @@ class Checkout(InterfaceView):
     def on_cancel_clicked(self, widget):
         self.close()
 
-    def on_ok_clicked(self, widget):
-        url = self.repositories.get_active_text()
-        path = self._get_path()
-        omit_externals = self.get_widget("omit_externals").get_active()
-        recursive = self.get_widget("recursive").get_active()
-        
-        if not url or not path:
-            rabbitvcs.ui.dialog.MessageBox(_("The repository URL and destination path are both required fields."))
-            return
-        
-        revision = self.revision_selector.get_revision_object()
-    
-        self.hide()
-        self.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs,
-            register_gtk_quit=self.gtk_quit_is_set()
-        )
-        self.action.append(self.action.set_header, _("Checkout"))
-        self.action.append(self.action.set_status, _("Running Checkout Command..."))
-        self.action.append(rabbitvcs.lib.helper.save_repository_path, url)
-        self.action.append(
-            self.vcs.checkout,
-            url,
-            path,
-            recurse=recursive,
-            revision=revision,
-            ignore_externals=omit_externals
-        )
-        self.action.append(self.action.set_status, _("Completed Checkout"))
-        self.action.append(self.action.finish)
-        self.action.start()
-
     def on_file_chooser_clicked(self, widget, data=None):
         chooser = rabbitvcs.ui.dialog.FolderChooser()
         path = chooser.run()
         if path is not None:
             self.get_widget("destination").set_text(path)
 
-    def on_repositories_changed(self, widget, data=None):
-        url = self.repositories.get_active_text()
-        tmp = url.replace("//", "/").split("/")[1:]
-        append = ""
-        prev = ""
-        while len(tmp):
-            prev = append
-            append = tmp.pop()
-            if append not in ("trunk", "branches", "tags"):
-                break
-                
-            if append in ("http:", "https:", "file:", "svn:", "svn+ssh:"):
-                append = ""
-                break
-                
-        self.get_widget("destination").set_text(
-            os.path.join(self.destination, append)
-        )
-        
-        self.check_form()
-
     def on_repositories_key_released(self, widget, data, userdata=None):
         if gtk.gdk.keyval_name(data.keyval) == "Return":
             if self.complete:
@@ -179,8 +113,8 @@ class Checkout(InterfaceView):
                 self.on_ok_clicked(widget)
 
     def on_repo_chooser_clicked(self, widget, data=None):
-        from rabbitvcs.ui.browser import BrowserDialog
-        BrowserDialog(self.repositories.get_active_text(), 
+        from rabbitvcs.ui.browser import SVNBrowserDialog
+        SVNBrowserDialog(self.repositories.get_active_text(), 
             callback=self.on_repo_chooser_closed)
 
     def on_repo_chooser_closed(self, new_url):
@@ -195,13 +129,124 @@ class Checkout(InterfaceView):
             self.complete = False
         
         self.get_widget("ok").set_sensitive(self.complete)
-        self.revision_selector.determine_widget_sensitivity()
+
+class SVNCheckout(Checkout):
+    def __init__(self, path=None, url=None, revision=None):
+        Checkout.__init__(self, path, url, revision)
+        self.get_widget("Checkout").set_title(_("Checkout - %s") % path)
+
+        self.svn = self.vcs.svn()
+
+        self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
+            self.get_widget("revision_container"),
+            self.svn,
+            revision=revision,
+            url_combobox=self.repositories,
+            expand=True
+        )
+
+        self.get_widget("options_box").show()
+        self.get_widget("revision_selector_box").show()
+
+        self.check_form()
+
+    def on_ok_clicked(self, widget):
+        url = self.repositories.get_active_text()
+        path = self._get_path()
+        omit_externals = self.get_widget("omit_externals").get_active()
+        recursive = self.get_widget("recursive").get_active()
+        
+        if not url or not path:
+            rabbitvcs.ui.dialog.MessageBox(_("The repository URL and destination path are both required fields."))
+            return
+        
+        revision = self.revision_selector.get_revision_object()
+    
+        self.hide()
+        self.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
+            register_gtk_quit=self.gtk_quit_is_set()
+        )
+        self.action.append(self.action.set_header, _("Checkout"))
+        self.action.append(self.action.set_status, _("Running Checkout Command..."))
+        self.action.append(rabbitvcs.util.helper.save_repository_path, url)
+        self.action.append(
+            self.svn.checkout,
+            rabbitvcs.util.helper.quote_url(url),
+            path,
+            recurse=recursive,
+            revision=revision,
+            ignore_externals=omit_externals
+        )
+        self.action.append(self.action.set_status, _("Completed Checkout"))
+        self.action.append(self.action.finish)
+        self.action.start()
+
+    def on_repositories_changed(self, widget, data=None):
+        # Do not use quoting for this bit
+        url = self.repositories.get_active_text()
+        tmp = url.replace("//", "/").split("/")[1:]
+        append = ""
+        prev = ""
+        while len(tmp):
+            prev = append
+            append = tmp.pop()
+            if append not in ("trunk", "branches", "tags"):
+                break
+                
+            if append in ("http:", "https:", "file:", "svn:", "svn+ssh:"):
+                append = ""
+                break
+        
+        self.get_widget("destination").set_text(
+            os.path.join(self.destination, append)
+        )
+        
+        self.check_form()
+
+class GitCheckout(GitUpdateToRevision):
+    def __init__(self, path, url, revision):
+        GitUpdateToRevision.__init__(self, path, revision)
+        self.get_widget("Update").set_title(_("Checkout - %s") % path)
+        self.get_widget("options_box").hide()
+
+class GitCheckoutQuiet:
+    def __init__(self, path):
+        self.vcs = rabbitvcs.vcs.VCS()
+        self.git = self.vcs.git(path)
+        self.action = rabbitvcs.ui.action.GitAction(
+            self.git,
+            run_in_thread=False
+        )
+
+        self.action.append(self.git.checkout, [path])
+        self.action.run()
+
+classes_map = {
+    rabbitvcs.vcs.VCS_SVN: SVNCheckout,
+    rabbitvcs.vcs.VCS_GIT: GitCheckout
+}
+
+def checkout_factory(vcs, path=None, url=None, revision=None, quiet=False):
+    if not vcs:
+        guess = rabbitvcs.vcs.guess(path)
+        vcs = guess["vcs"]
+
+    if vcs == rabbitvcs.vcs.VCS_DUMMY:
+        return SVNCheckout(path, url, revision)
+    elif vcs == rabbitvcs.vcs.VCS_GIT:
+        if quiet:
+            return GitCheckoutQuiet(path)
+        else:
+            return GitCheckout(path, url, revision)
+    
+    return classes_map[vcs](path, url, revision)
 
 if __name__ == "__main__":
-    from rabbitvcs.ui import main, REVISION_OPT
+    from rabbitvcs.ui import main, REVISION_OPT, VCS_OPT, QUIET_OPT
     (options, args) = main(
-        [REVISION_OPT],
-        usage="Usage: rabbitvcs checkout [url] [path]"
+        [REVISION_OPT, VCS_OPT, QUIET_OPT],
+        usage="Usage: rabbitvcs checkout --vcs=[git|svn] [url] [path]"
     )
     
     # If two arguments are passed:
@@ -220,6 +265,9 @@ if __name__ == "__main__":
         else:
             url = args[0]
 
-    window = Checkout(path=path, url=url, revision=options.revision)
-    window.register_gtk_quit()
-    gtk.main()
+    if options.quiet:
+        window = checkout_factory(options.vcs, path=path, url=url, revision=options.revision, quiet=options.quiet)
+    else:
+        window = checkout_factory(options.vcs, path=path, url=url, revision=options.revision, quiet=options.quiet)
+        window.register_gtk_quit()
+        gtk.main()
