@@ -4,7 +4,7 @@
 # 
 # Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
 # Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
-# Copyright (C) 2008-2008 by Adam Plumb <adamplumb@gmail.com>
+# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
 # 
 # RabbitVCS is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,30 +25,30 @@ import gobject
 import gtk
 
 from rabbitvcs.ui import InterfaceView
-from rabbitvcs.ui.action import VCSAction
+from rabbitvcs.ui.action import SVNAction
 import rabbitvcs.ui.widget
 import rabbitvcs.ui.dialog
-import rabbitvcs.lib.helper
+import rabbitvcs.util.helper
 
 from rabbitvcs import gettext
 _ = gettext.gettext
 
-class Import(InterfaceView):
+class SVNImport(InterfaceView):
     def __init__(self, path):
         InterfaceView.__init__(self, "import", "Import")
         
-        
         self.get_widget("Import").set_title(_("Import - %s") % path)
         
         self.path = path
-        self.vcs = rabbitvcs.lib.vcs.create_vcs_instance()
+        self.vcs = rabbitvcs.vcs.VCS()
+        self.svn = self.vcs.svn()
         
-        if self.vcs.is_in_a_or_a_working_copy(path):
-            self.get_widget("repository").set_text(self.vcs.get_repo_url(path))
+        if self.svn.is_in_a_or_a_working_copy(path):
+            self.get_widget("repository").set_text(self.svn.get_repo_url(path))
         
         self.repositories = rabbitvcs.ui.widget.ComboBox(
             self.get_widget("repositories"), 
-            rabbitvcs.lib.helper.get_repository_paths()
+            rabbitvcs.util.helper.get_repository_paths()
         )
         
         self.message = rabbitvcs.ui.widget.TextView(
@@ -72,15 +72,15 @@ class Import(InterfaceView):
         
         self.hide()
 
-        self.action = rabbitvcs.ui.action.VCSAction(
-            self.vcs,
+        self.action = rabbitvcs.ui.action.SVNAction(
+            self.svn,
             register_gtk_quit=self.gtk_quit_is_set()
         )
         
         self.action.append(self.action.set_header, _("Import"))
         self.action.append(self.action.set_status, _("Running Import Command..."))
         self.action.append(
-            self.vcs.import_, 
+            self.svn.import_, 
             self.path,
             url,
             self.message.get_text(),
@@ -96,10 +96,18 @@ class Import(InterfaceView):
         if message is not None:
             self.message.set_text(message)
 
+classes_map = {
+    rabbitvcs.vcs.VCS_SVN: SVNImport
+}
+
+def import_factory(path):
+    vcs = rabbitvcs.vcs.VCS_SVN    
+    return classes_map[vcs](path)
+
 if __name__ == "__main__":
     from rabbitvcs.ui import main
     (options, paths) = main(usage="Usage: rabbitvcs import [path]")
             
-    window = Import(paths[0])
+    window = import_factory(paths[0])
     window.register_gtk_quit()
     gtk.main()
