@@ -1,20 +1,21 @@
 import locale
 import os
 
-from rabbitvcs.lib.log import Log
+from rabbitvcs.util.log import Log
 log = Log("rabbitvcs.util.locale")
 
 def initialize_locale():
     try:
+        sane_default = locale.getdefaultlocale(['LANG', 'LANGUAGE'])
         # Just try to set the default locale for the user
-        locale.resetlocale(locale.LC_ALL)
+        locale.setlocale(locale.LC_ALL, sane_default)
     except locale.Error:
         # If the user's environment does not specify an encoding, Python will
         # pick a default which might not be available. It seems to pick
         # ISO8859-1 (latin1), but UTF8 is a better idea on GNU/Linux.
         log.warning("Could not set default locale (LANG: %s)" % os.environ.get("LANG"))
         
-        (loc, enc) = locale.getdefaultlocale()
+        (loc, enc) = sane_default
         
         # We should only try this if we have a region to set as well.
         if loc and enc != "UTF8":
